#!/bin/bash
# Copyright 2016 Michal Žejdl
# snmpget enterprises.43 OID(s) and store to RRD

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2

#echo "$(date '+%F %X') $(basename $0) $address $agent"

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

# group write permissions for rrdcached
umask 002

rrdstep=$snmpinterval
rrdheartbeat=$((2 * rrdstep))
rraxff=0.5
rrathirtym=$((30 * 60 / rrdstep))
rratwoh=$((4 * rrathirtym))
rraday=$((12 * rratwoh))
rraweek=$((7 * rraday))
rrarows=600

rrddestdir=$rrddir/snmpagent/$agent/3Com
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# no retry (-r 0), do not resend request if OID is missing (-Cf), exit on error
result=($(snmpget -m '' -v 1 -r 0 -c $snmprcommunity -Cf -Onqt $address \
.1.3.6.1.4.1.43.45.1.6.1.1.1.4.65536 \
.1.3.6.1.4.1.43.45.1.6.1.2.1.1.2.65536 \
.1.3.6.1.4.1.43.45.1.6.1.2.1.1.3.65536 \
.1.3.6.1.4.1.43.45.1.6.1.2.1.1.4.65536 \
))

snmpget_status=$?
# leave if something goes wrong
test $snmpget_status -eq 0 || exit $snmpget_status

IFS=$oldifs

# prepare for inserting into myslq after incomplete response
hwCpuCostRatePer5Min=0
hwMemSize=0
hwMemFree=0
hwMemRawSliceUsed=0
# ugly sanitizing
for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
	if test ${result[oid_row]::36} = ".1.3.6.1.4.1.43.45.1.6.1.1.1.4.65536"
	then
		hwCpuCostRatePer5Min=${result[$oid_row]#.1.3.6.1.4.1.43.45.1.6.1.1.1.4.65536 }
		is_natural_number "$hwCpuCostRatePer5Min" || hwCpuCostRatePer5Min=0
	else
		case ${result[oid_row]::38} in
		.1.3.6.1.4.1.43.45.1.6.1.2.1.1.2.65536)
			hwMemSize=${result[$oid_row]#.1.3.6.1.4.1.43.45.1.6.1.2.1.1.2.65536 }
			is_natural_number "$hwMemSize" || hwMemSize=0
			;;
		.1.3.6.1.4.1.43.45.1.6.1.2.1.1.3.65536)
			hwMemFree=${result[$oid_row]#.1.3.6.1.4.1.43.45.1.6.1.2.1.1.3.65536 }
			is_natural_number "$hwMemFree" || hwMemFree=0
			;;
		.1.3.6.1.4.1.43.45.1.6.1.2.1.1.4.65536)
			hwMemRawSliceUsed=${result[$oid_row]#.1.3.6.1.4.1.43.45.1.6.1.2.1.1.4.65536 }
			is_natural_number "$hwMemRawSliceUsed" || hwMemRawSliceUsed=0
			;;
		esac
	fi
done

# debugging
#echo "$(date '+%F %X') $(basename $0) address: $address hwCpuCostRatePer5Min: $hwCpuCostRatePer5Min hwMemSize $hwMemSize hwMemFree: $hwMemFree hwMemRawSliceUsed: $hwMemRawSliceUsed"
#echo "$(date '+%F %X') $(basename $0) hwCpuCostRatePer5Min: $hwCpuCostRatePer5Min hwMemSize $hwMemSize hwMemFree: $hwMemFree hwMemRawSliceUsed: $hwMemRawSliceUsed"

rrdfile="$rrddestdir/hwDevice.rrd"

# --start is not necessary
# DS can be max 19 chars long (hwCpuCostRatePer5Mi)
test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:hwCpuCostRatePer5Mi:GAUGE:$rrdheartbeat:0:U \
DS:hwMemSize:GAUGE:$rrdheartbeat:0:U \
DS:hwMemFree:GAUGE:$rrdheartbeat:0:U \
DS:hwMemRawSliceUsed:GAUGE:$rrdheartbeat:0:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

rrdtool update "$rrdfile" $rrdcached N:\
$hwCpuCostRatePer5Min:\
$hwMemSize:\
$hwMemFree:\
$hwMemRawSliceUsed
