#!/bin/bash
# Copyright 2018 Michal Žejdl
# snmpget some enterprises.12140 OIDs and store them to the RRD file

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

functions=$coredir/snmp/functions.sh
test -f $functions || exit
. $functions

address=$1
agent=$2

# group write permissions for rrdcached
umask 002

rrdstep=$snmpinterval
rrdheartbeat=$((2 * rrdstep))
rraxff=0.5
rrathirtym=$((30 * 60 / rrdstep))
rratwoh=$((4 * rrathirtym))
rraday=$((12 * rratwoh))
rraweek=$((7 * rraday))
rrarows=600

rrddestdir=$rrddir/snmpagent/$agent/Alcoma
test -d "$rrddestdir" || mkdir -p "$rrddestdir" || exit

oldifs=$IFS
# one line = one array element; and focus only on the first line
IFS=$'\n'
# no retry (-r 0), do not resend request if OID is missing (-Cf), exit on error
#.1.3.6.1.4.1.12140.2.9.10.0 \
result=($(snmpget -m '' -v 1 -r 0 -c $snmprcommunity -Cf -Onqt $address \
.1.3.6.1.4.1.12140.2.3.4.0 \
.1.3.6.1.4.1.12140.2.3.9.0 \
.1.3.6.1.4.1.12140.2.4.1.0 \
.1.3.6.1.4.1.12140.2.4.2.0 \
.1.3.6.1.4.1.12140.2.9.7.0 \
.1.3.6.1.4.1.12140.2.9.8.0 \
.1.3.6.1.4.1.12140.2.9.9.0 \
.1.3.6.1.4.1.12140.2.9.10.0 \
.1.3.6.1.4.1.12140.2.10.7.0 \
.1.3.6.1.4.1.12140.2.10.8.0 \
.1.3.6.1.4.1.12140.2.10.9.0 \
.1.3.6.1.4.1.12140.2.10.10.0 \
.1.3.6.1.4.1.12140.2.11.7.0 \
.1.3.6.1.4.1.12140.2.11.8.0 \
.1.3.6.1.4.1.12140.2.11.9.0 \
.1.3.6.1.4.1.12140.2.11.10.0 \
))

snmpget_status=$?
# leave if something goes wrong
test $snmpget_status -eq 0 || exit $snmpget_status

IFS=$oldifs

# init variables for inserting into the RRD even if the values won't be returned
alMPRXLevel=U
alMPTemperature=U
alMPQuality=U
alMPSNR=U
alMPPktRX2=U
alMPByteRX2=U
alMPPktTX2=U
alMPByteTX2=U
alMPPktRX3=U
alMPByteRX3=U
alMPPktTX3=U
alMPByteTX3=U
alMPPktRX4=U
alMPByteRX4=U
alMPPktTX4=U
alMPByteTX4=U

# ugly sanitizing
for ((oid_row = 0; oid_row < ${#result[@]}; oid_row++)); do
	case ${result[oid_row]%% *} in
	.1.3.6.1.4.1.12140.2.3.4.0)
		alMPRXLevel=${result[$oid_row]#.1.3.6.1.4.1.12140.2.3.4.0 }
		is_number "$alMPRXLevel" || alMPRXLevel=U
		;;
	.1.3.6.1.4.1.12140.2.3.9.0)
		alMPTemperature=${result[$oid_row]#.1.3.6.1.4.1.12140.2.3.9.0 }
		is_number "$alMPTemperature" || alMPTemperature=U
		;;
	.1.3.6.1.4.1.12140.2.4.1.0)
		alMPQuality=${result[$oid_row]#.1.3.6.1.4.1.12140.2.4.1.0 }
		is_number "$alMPQuality" || alMPQuality=U
		;;
	.1.3.6.1.4.1.12140.2.4.2.0)
		# without a MIB (-m '') there are quotes around the string
		alMPSNR=${result[$oid_row]#.1.3.6.1.4.1.12140.2.4.2.0 \"}
		alMPSNR=${alMPSNR%\"}
		replace_specchars "$alMPSNR" && alMPSNR=$without_specchars
		;;
	.1.3.6.1.4.1.12140.2.9.7.0)
		alMPPktRX2=${result[$oid_row]#.1.3.6.1.4.1.12140.2.9.7.0 }
		is_natural_number "$alMPPktRX2" || alMPPktRX2=U
		;;
	.1.3.6.1.4.1.12140.2.9.8.0)
		alMPByteRX2=${result[$oid_row]#.1.3.6.1.4.1.12140.2.9.8.0 }
		is_natural_number "$alMPByteRX2" || alMPByteRX2=U
		;;
	.1.3.6.1.4.1.12140.2.9.9.0)
		alMPPktTX2=${result[$oid_row]#.1.3.6.1.4.1.12140.2.9.9.0 }
		is_natural_number "$alMPPktTX2" || alMPPktTX2=U
		;;
	.1.3.6.1.4.1.12140.2.9.10.0)
		alMPByteTX2=${result[$oid_row]#.1.3.6.1.4.1.12140.2.9.10.0 }
		is_natural_number "$alMPByteTX2" || alMPByteTX2=U
		;;
	.1.3.6.1.4.1.12140.2.10.7.0)
		alMPPktRX3=${result[$oid_row]#.1.3.6.1.4.1.12140.2.10.7.0 }
		is_natural_number "$alMPPktRX3" || alMPPktRX3=U
		;;
	.1.3.6.1.4.1.12140.2.10.8.0)
		alMPByteRX3=${result[$oid_row]#.1.3.6.1.4.1.12140.2.10.8.0 }
		is_natural_number "$alMPByteRX3" || alMPByteRX3=U
		;;
	.1.3.6.1.4.1.12140.2.10.9.0)
		alMPPktTX3=${result[$oid_row]#.1.3.6.1.4.1.12140.2.10.9.0 }
		is_natural_number "$alMPPktTX3" || alMPPktTX3=U
		;;
	.1.3.6.1.4.1.12140.2.10.10.0)
		alMPByteTX3=${result[$oid_row]#.1.3.6.1.4.1.12140.2.10.10.0 }
		is_natural_number "$alMPByteTX3" || alMPByteTX3=U
		;;
	.1.3.6.1.4.1.12140.2.11.7.0)
		alMPPktRX4=${result[$oid_row]#.1.3.6.1.4.1.12140.2.11.7.0 }
		is_natural_number "$alMPPktRX4" || alMPPktRX4=U
		;;
	.1.3.6.1.4.1.12140.2.11.8.0)
		alMPByteRX4=${result[$oid_row]#.1.3.6.1.4.1.12140.2.11.8.0 }
		is_natural_number "$alMPByteRX4" || alMPByteRX4=U
		;;
	.1.3.6.1.4.1.12140.2.11.9.0)
		alMPPktTX4=${result[$oid_row]#.1.3.6.1.4.1.12140.2.11.9.0 }
		is_natural_number "$alMPPktTX4" || alMPPktTX4=U
		;;
	.1.3.6.1.4.1.12140.2.11.10.0)
		alMPByteTX4=${result[$oid_row]#.1.3.6.1.4.1.12140.2.11.10.0 }
		is_natural_number "$alMPByteTX4" || alMPByteTX4=U
		;;
	esac
done

# debugging
#echo "$(date +'%F %X') $(basename $0) alMPRXLevel: $alMPRXLevel, alMPTemperature: $alMPTemperature, alMPQuality: $alMPQuality, alMPSNR: $alMPSNR, alMPPktRX2: $alMPPktRX2, alMPByteRX2: $alMPByteRX2, alMPPktTX2: $alMPPktTX2, alMPByteTX2: $alMPByteTX2, alMPPktRX3: $alMPPktRX3, alMPByteRX3: $alMPByteRX3, alMPPktTX3: $alMPPktTX3, alMPByteTX3: $alMPByteTX3, alMPPktRX4: $alMPPktRX4, alMPByteRX4: $alMPByteRX4, alMPPktTX4: $alMPPktTX4, alMPByteTX4: $alMPByteTX4"


# alMPODU

# check if we got at last one value
if test "$alMPRXLevel" != U -o "$alMPTemperature" != U
then
	rrdfile="$rrddestdir/alMPODU.rrd"
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:RXLevel:GAUGE:$rrdheartbeat:U:U \
DS:Temperature:GAUGE:$rrdheartbeat:U:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:"$alMPRXLevel":"$alMPTemperature"
fi


# alMPModem

# check if we got at last one value
if test "$alMPQuality" != U -o "$alMPSNR" != U
then
	rrdfile="$rrddestdir/alMPModem.rrd"
	test -f "$rrdfile" || rrdtool create "$rrdfile" --step $rrdstep \
DS:Quality:GAUGE:$rrdheartbeat:U:U \
DS:SNR:GAUGE:$rrdheartbeat:U:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

	rrdtool update "$rrdfile" $rrdcached N:"$alMPQuality":"$alMPSNR"
fi


# alMPLine2 alMPLine3 alMPLine4

for line in 2 3 4
do
	eval alMPPktRX=\$alMPPktRX$line
	eval alMPByteRX=\$alMPByteRX$line
	eval alMPPktTX=\$alMPPktTX$line
	eval alMPByteTX=\$alMPByteTX$line

	# check if we got at last one value
	if test "$alMPPktRX" != U -o "$alMPByteRX" != U \
			-o "$alMPPktTX" != U -o "$alMPByteTX" != U
	then
		rrdfile="$rrddestdir/alMPLine$line.rrd"
		test -f "$rrdfile" || rrdtool create "$rrdfile" \
				--step $rrdstep \
DS:PktRX:COUNTER:$rrdheartbeat:0:U \
DS:ByteRX:COUNTER:$rrdheartbeat:0:U \
DS:PktTX:COUNTER:$rrdheartbeat:0:U \
DS:ByteTX:COUNTER:$rrdheartbeat:0:U \
RRA:AVERAGE:$rraxff:1:$rrarows \
RRA:AVERAGE:$rraxff:$rrathirtym:$rrarows \
RRA:AVERAGE:$rraxff:$rratwoh:$rrarows \
RRA:AVERAGE:$rraxff:$rraday:$rrarows \
RRA:AVERAGE:$rraxff:$rraweek:$rrarows \
RRA:MIN:$rraxff:1:$rrarows \
RRA:MIN:$rraxff:$rrathirtym:$rrarows \
RRA:MIN:$rraxff:$rratwoh:$rrarows \
RRA:MIN:$rraxff:$rraday:$rrarows \
RRA:MIN:$rraxff:$rraweek:$rrarows \
RRA:MAX:$rraxff:1:$rrarows \
RRA:MAX:$rraxff:$rrathirtym:$rrarows \
RRA:MAX:$rraxff:$rratwoh:$rrarows \
RRA:MAX:$rraxff:$rraday:$rrarows \
RRA:MAX:$rraxff:$rraweek:$rrarows \
RRA:LAST:$rraxff:1:$rrarows \
RRA:LAST:$rraxff:$rrathirtym:$rrarows \
RRA:LAST:$rraxff:$rratwoh:$rrarows \
RRA:LAST:$rraxff:$rraday:$rrarows \
RRA:LAST:$rraxff:$rraweek:$rrarows

		rrdtool update "$rrdfile" $rrdcached N:\
$alMPPktRX:\
$alMPByteRX:\
$alMPPktTX:\
$alMPByteTX
	fi
done
