#!/bin/bash
# Copyright 2008 - 2017 Michal Žejdl
# based on obsolete fdb.sh
# called from collector.sh
# it reads SNMP FDB of given node and fills the MySQL table mibfdbport
# input: node (arg), checked.hasfdb (sql)
# dependence: dirname, mysql, cut, snmpwalk, gawk

confdir=$(dirname $0)/../..
config=$confdir/defaults.sh
test -f $config || exit
. $config

address=$1
agent=$2
organization=$3
sysObjectID=$4

# a new file descriptor to prevent pipe from closing
exec 3> $mysqlwpipe

# OIDs
dot1dBasePortIfIndex=".1.3.6.1.2.1.17.1.4.1.2"
# bridge FDB - ports (with MAC addresses as oids)
dot1dTpFdbPort=".1.3.6.1.2.1.17.4.3.1.2"

vlans=""

case $organization in
	9)
		# Cisco
		vlans=$(snmpwalk -Cc -m '' -v 1 -c $snmprcommunity -Onq $address .1.3.6.1.4.1.9.9.46.1.3.1.1.2 2>> $logdir/fdb-snmpwalk.err | gawk -F[.[:space:]] '/^\.1\.3\.6\.1\.4\.1\.9\.9\.46\.1\.3\.1\.1\.2\.1\.[[:digit:]]+[[:space:]]1$/ {print $17}'; exit ${PIPESTATUS[0]})
		snmpget_status=$?
		;;
	43)
		# 3Com
		case $sysObjectID in
		.1.3.6.1.4.1.43.10.27.4.1.2.1 | .1.3.6.1.4.1.43.10.27.4.1.2.2)
			# Switch 1100 or 3300
			vlans=$(snmpwalk -Cc -m '' -v 1 -c $snmprcommunity -Onq $address .1.3.6.1.4.1.43.10.1.14.1.1 2>> $logdir/fdb-snmpwalk.err | gawk -F[.[:space:]] '/^\.1\.3\.6\.1\.4\.1\.43\.10\.1\.14\.1\.1\.1\.2\.[[:digit:]]+[[:space:]][[:digit:]]+$/ {print $16}'; exit ${PIPESTATUS[0]})
			snmpget_status=$?
			#echo "$(date '+%F %X') $(basename $0) $address $sysObjectID $vlans"
			;;
		esac
		;;
esac

# assume that snmpwalk above never returns "null" in $vlan
# but if so, nothing bad happens
# - this will only lead into snmpwalk without vlan reading
test -z "$vlans" && vlans="null"

for vlan in $vlans; do

	if test "$vlan" = "null"; then
		community=$snmprcommunity
	else
		# to get FDB by iterating <community name>@<VLAN number>
		community=$snmprcommunity@$vlan
	fi

	# fill the mibportifindex table
	# snmpwalk - numeric oids, only values (integer), TO to log file
	# gawk - enable intervals in regexps for ver. 3, input checking
	snmpwalk -Cc -m '' -v 1 -c $community -Onq $address $dot1dBasePortIfIndex 2>> $logdir/snmp/$(basename $0 .sh).err | gawk -F[.[:space:]] -v agent=$agent -v vlan=$vlan --re-interval '/^\.1\.3\.6\.1\.2\.1\.17\.1\.4\.1\.2\.[[:digit:]]+[[:space:]][[:digit:]]+$/ {printf "insert mibportifindex (agent, time, vlan, port, ifIndex) values (%d, unix_timestamp(), %s, %d, %d) on duplicate key update time = unix_timestamp(), vlan = %s, ifIndex = %d;\n", agent, vlan, $13, $14, vlan, $14; fflush()}' >&3
	snmpget_status=${PIPESTATUS[0]}

	# snmpwalk - numeric oids, only values (integer), TO to log file
	# gawk - enable intervals in regexps for ver. 3, input checking
	snmpwalk -Cc -m '' -v 1 -c $community -Onq $address $dot1dTpFdbPort 2>> $logdir/snmp/$(basename $0 .sh).err | gawk -F[.[:space:]] -v agent=$agent -v vlan=$vlan --re-interval '/^\.1\.3\.6\.1\.2\.1\.17\.4\.3\.1\.2(\.[[:digit:]]{1,3}){6}[[:space:]][[:digit:]]+$/ {printf "insert mibfdbport (agent, time, vlan, mac, port) values (%d, unix_timestamp(), %s, \47%02x:%02x:%02x:%02x:%02x:%02x\47, %d) on duplicate key update time = unix_timestamp(), vlan = %s, port = %d;\n", agent, vlan, $13, $14, $15, $16, $17, $18, $19, vlan, $19; fflush()}' >&3
	snmpget_status=${PIPESTATUS[0]}

done

# pass the exit status of the last snmp* command to the collector.sh
exit $snmpget_status
